import UploadFile from '@/components/upload-file'
import deepClone from '@/utils/deep.clone'
import { NForm, NInput, NSpace, NButton, useMessage, NFormItem } from 'naive-ui'
import { defineComponent, onMounted, PropType, reactive, ref } from 'vue'
import { subjectSave, subjectUpdate } from '../api'
export default defineComponent({
  name: 'role-operation',
  props: {
    type: {
      type: String,
      default: 'add'
    },
    applyList: {
      type: Array as PropType<any>,
      default: () => []
    },
    data: {
      type: Object as PropType<any>,
      default: () => {}
    }
  },
  emits: ['close', 'getList'],
  setup(props, { slots, attrs, emit }) {
    const forms = reactive({
      name: null,
      code: null,
      img: null,
      parentId: 0
    })
    const btnLoading = ref(false)
    const formsRef = ref()
    const message = useMessage()

    const onSubmit = async () => {
      formsRef.value.validate(async (error: any) => {
        if (error) return false
        try {
          btnLoading.value = true

          if (props.type === 'add') {
            await subjectSave({ ...forms })
            message.success('添加成功')
          } else if (props.type === 'edit') {
            await subjectUpdate({
              ...forms,
              id: props.data.id
            })
            message.success('修改成功')
          }

          emit('close')
          emit('getList')
        } catch {}
        btnLoading.value = false
      })
    }

    onMounted(async () => {
      if (props.type === 'edit') {
        const data = props.data
        forms.img = data.img
        forms.name = data.name
        forms.code = data.code
      }
    })

    return () => (
      <div style="background: #fff; padding-top: 12px">
        <NForm model={forms} ref={formsRef} label-placement="left" label-width="auto">
          <NFormItem
            label="声部图片"
            path="img"
            rule={[
              {
                required: true,
                message: '请输入声部图片'
              }
            ]}
          >
            <UploadFile
              size={2}
              cropper
              v-model:fileList={forms.img}
              bucketName="gyt"
              path="basic/"
              tips="图片大小2M以内"
            />
          </NFormItem>
          <NFormItem
            label="声部名称"
            path="name"
            rule={[
              {
                required: true,
                message: '请输入声部名称'
              }
            ]}
          >
            <NInput
              v-model:value={forms.name}
              placeholder="请输入声部名称"
              clearable
              maxlength={100}
            ></NInput>
          </NFormItem>
          <NFormItem label="声部编码" path="code">
            <NInput v-model:value={forms.code} placeholder="请输入声部编码" clearable></NInput>
          </NFormItem>
        </NForm>

        <NSpace justify="end">
          <NButton type="default" onClick={() => emit('close')}>
            取消
          </NButton>
          <NButton type="primary" onClick={() => onSubmit()} loading={btnLoading.value}>
            保存
          </NButton>
        </NSpace>
      </div>
    )
  }
})
